package org.eclipse.wtp.releng.tools;

import java.net.URI;
import java.net.URISyntaxException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.equinox.internal.p2.artifact.repository.simple.SimpleArtifactRepositoryFactory;
import org.eclipse.equinox.p2.core.ProvisionException;
import org.eclipse.equinox.p2.publisher.IPublisherAction;
import org.eclipse.equinox.p2.publisher.IPublisherInfo;
import org.eclipse.equinox.p2.publisher.Publisher;
import org.eclipse.equinox.p2.publisher.PublisherInfo;
import org.eclipse.equinox.p2.repository.IRepository;
import org.eclipse.equinox.p2.repository.IRepositoryManager;
import org.eclipse.equinox.p2.repository.artifact.IArtifactRepository;

public class AddMirrorsURL extends Task {

    private static final String IREPOSITORY_P2_STATS_URI = "p2.statsURI";
    private static final String ARTIFACT_REPO_DIRECTORY  = "artifactRepoDirectory";
    private static final String P2_MIRRORS_URL           = "p2MirrorsURL";
    private static final String P2_STATS_URI             = "p2StatsURI";
    private String              artifactRepoDirectory;
    private String              p2MirrorsURL;
    private String              p2StatsURI;

    public void rewriteMirrorURL() throws ProvisionException, URISyntaxException {
        try {
            IPublisherInfo info = createPublisherInfo();
            if (info != null) {
                IPublisherAction[] actions = createActions();
                Publisher publisher = new Publisher(info);
                publisher.publish(actions, new NullProgressMonitor());
                infoOutput("artifact repository mirrorURL rewritten");
            }
            else {
                infoOutput("artifact repository was not rewritten");
            }
        }
        catch (ProvisionException e) {
            infoOutput(e.getMessage());
            throw e;
        }
        catch (URISyntaxException e) {
            infoOutput(e.getMessage());
            throw e;
        }
    }

    private IPublisherInfo createPublisherInfo() throws ProvisionException, URISyntaxException {
        PublisherInfo result = null;

        String pathPart = getArtifactRepoDirectory();
        if ((pathPart == null) || (pathPart.length() == 0)) {
            infoOutput("The repository directory needs to be specified for this task. Try setting system property '" + ARTIFACT_REPO_DIRECTORY + "'?");
        }
        else {
            String repoURI = "file:/" + pathPart;


            SimpleArtifactRepositoryFactory simpleArtifactRepositoryFactory = new SimpleArtifactRepositoryFactory();
            // NPE is thrown during "reload" if repository is written, and
            // agent is not set
            simpleArtifactRepositoryFactory.setAgent(Activator.getProvisioningAgent());
            IArtifactRepository artifactRepository = simpleArtifactRepositoryFactory.load(new URI(repoURI), IRepositoryManager.REPOSITORY_HINT_MODIFIABLE, new NullProgressMonitor());
            if (artifactRepository != null) {
                result = processMirrorsURL(result, artifactRepository);
                result = processStatsURI(result, artifactRepository);
            }
            else {
                infoOutput("repository was null, not local, or otherwise not modifiable");
            }
        }
        return result;
    }

    private PublisherInfo processMirrorsURL(PublisherInfo result, IArtifactRepository artifactRepository) {

        String existingMirrorURL = artifactRepository.getProperty(IRepository.PROP_MIRRORS_URL);
        if (existingMirrorURL != null) {
            infoOutput("the repository had existing mirrorURL: " + existingMirrorURL);
        }
        else {
            infoOutput("the repository had no existing mirrorURL");
        }

        String newMirrorURL = getP2MirrorsURL();
        // if property given to us is empty string, assume that is
        // the same as "removal" which is done by passing in null.
        if ((newMirrorURL != null) && (newMirrorURL.length() == 0)) {
            newMirrorURL = null;
        }
        boolean needrewrite = false;
        String reasonForNoWrite = null;
        if (newMirrorURL != null) {
            // if they are the same, don't bother
            if (!newMirrorURL.equals(existingMirrorURL)) {
                needrewrite = true;
            }
            else {
                reasonForNoWrite = "new value and existing value of mirrorURL are the same";
            }
        }
        else {
            // newMirror is null, means removal is desired
            if (existingMirrorURL != null) {
                needrewrite = true;
            }
            else {
                reasonForNoWrite = "removal of mirrorURL was indicated, but repository has no existing value.";
            }
        }
        if (needrewrite) {
            if (result == null) {
                result = initRepoOverwrite(artifactRepository);
            }
            artifactRepository.setProperty(IRepository.PROP_MIRRORS_URL, newMirrorURL);
        }
        else {
            infoOutput(reasonForNoWrite);
        }
        return result;
    }

    private IPublisherAction[] createActions() {
        IPublisherAction[] result = new IPublisherAction[0];
        return result;
    }

    public String getArtifactRepoDirectory() {
        // if null, that is not explicitly 'set', then see if it was
        // passed in as a system property. TODO: Does this contradict
        // ant task rules?
        if (artifactRepoDirectory == null) {
            // for now, be explicit we are working with "artifact"
            // repositories ... but could also do metadata and/or accept other
            // input forms, such as p2.buildRepo.
            artifactRepoDirectory = System.getProperty(ARTIFACT_REPO_DIRECTORY);
            infoOutput("repository directory set from '" + ARTIFACT_REPO_DIRECTORY + "': " + artifactRepoDirectory);
        }
        return artifactRepoDirectory;
    }

    private void infoOutput(String info) {
        System.out.println("\t" + info);
    }

    public void setArtifactRepoDirectory(String repositoryDirectory) {
        this.artifactRepoDirectory = repositoryDirectory;
    }

    public String getP2MirrorsURL() {
        if (p2MirrorsURL == null) {
            p2MirrorsURL = System.getProperty(P2_MIRRORS_URL);
            infoOutput("mirror URL value set from '" + P2_MIRRORS_URL + "': " + p2MirrorsURL);

        }
        return p2MirrorsURL;
    }

    public void setP2MirrorsURL(String mirrorURLString) {
        this.p2MirrorsURL = mirrorURLString;
    }

    public void setP2StatsURI(String statsURIString) {
        this.p2StatsURI = statsURIString;
    }

    @Override
    public void execute() throws BuildException {
        try {
            rewriteMirrorURL();
        }
        catch (ProvisionException e) {
            // e.printStackTrace();
            throw new BuildException(e);
        }
        catch (URISyntaxException e) {

            e.printStackTrace();
            throw new BuildException(e);
        }
    }

    private PublisherInfo processStatsURI(PublisherInfo result, IArtifactRepository artifactRepository) {

        /*
         * <property name='p2.statsURI'
         * value='http://your.stats.server/stats'/>
         */
        String existingStatsURI = artifactRepository.getProperty(IREPOSITORY_P2_STATS_URI);
        if (existingStatsURI != null) {
            infoOutput("the repository had existing statsURI: " + existingStatsURI);
        }
        else {
            infoOutput("the repository had no existing statsURI");
        }

        String newStatsURI = getP2StatsURI();
        // if property given to us is empty string, assume that is
        // the same as "removal" which is done by passing in null.
        if ((newStatsURI != null) && (newStatsURI.length() == 0)) {
            newStatsURI = null;
        }
        boolean needrewrite = false;
        String reasonForNoWrite = null;
        if (newStatsURI != null) {
            // if they are the same, don't bother
            if (!newStatsURI.equals(existingStatsURI)) {
                needrewrite = true;
            }
            else {
                reasonForNoWrite = "new value and existing value of statsURI are the same";
            }
        }
        else {
            // newStatsURI is null, means removal is desired
            if (existingStatsURI != null) {
                needrewrite = true;
            }
            else {
                reasonForNoWrite = "removal of statusURI was indicated, but repository has no existing value.";
            }
        }
        if (needrewrite) {
            if (result == null) {
                result = initRepoOverwrite(artifactRepository);
            }
            artifactRepository.setProperty(IREPOSITORY_P2_STATS_URI, newStatsURI);

        }
        else {
            infoOutput(reasonForNoWrite);
        }
        return result;
    }

    public String getP2StatsURI() {
        if (p2StatsURI == null) {
            p2StatsURI = System.getProperty(P2_STATS_URI);
            infoOutput("stats URI value set from '" + P2_STATS_URI + "': " + p2StatsURI);

        }
        return p2StatsURI;
    }

    private PublisherInfo initRepoOverwrite(IArtifactRepository artifactRepository) {
        PublisherInfo result;
        result = new PublisherInfo();
        result.setArtifactRepository(artifactRepository);
        result.setArtifactOptions(IPublisherInfo.A_OVERWRITE);
        return result;
    }

}
